@@ -5,6 +5,176 @@
 
 setGeneric("gsva", function(expr, gset.idx.list, ...) standardGeneric("gsva"))
 
+setMethod("gsva", signature(expr="HDF5Array", gset.idx.list="list"),
+          function(expr, gset.idx.list, annotation,
+  method=c("gsva", "ssgsea", "zscore", "plage"),
+  kcdf=c("Gaussian", "Poisson", "none"),
+  abs.ranking=FALSE,
+  min.sz=1,
+  max.sz=Inf,
+  parallel.sz=1L,
+  mx.diff=TRUE,
+  tau=switch(method, gsva=1, ssgsea=0.25, NA),
+  ssgsea.norm=TRUE,
+  verbose=TRUE,
+  BPPARAM=SerialParam(progressbar=verbose))
+{
+  method <- match.arg(method)
+  kcdf <- match.arg(kcdf)
+  
+  warning("Using 'HDF5Array' objects as input is still in an experimental stage.")
+
+  ## filter genes according to verious criteria,
+  ## e.g., constant expression
+  expr <- .filterFeatures(expr, method)
+  
+  ## map to the actual features for which expression data is available
+  mapped.gset.idx.list <- .mapGeneSetsToFeatures(gset.idx.list, rownames(expr))
+  
+  ## remove gene sets from the analysis for which no features are available
+  ## and meet the minimum and maximum gene-set size specified by the user
+  mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
+                                         min.sz=max(1, min.sz),
+                                         max.sz=max.sz)
+  
+  if (!missing(kcdf)) {
+    if (kcdf == "Gaussian") {
+      rnaseq <- FALSE
+      kernel <- TRUE
+    } else if (kcdf == "Poisson") {
+      rnaseq <- TRUE
+      kernel <- TRUE
+    } else
+      kernel <- FALSE
+  }
+  
+  rval <- .gsva(expr, mapped.gset.idx.list, method, kcdf, rnaseq, abs.ranking,
+                parallel.sz, mx.diff, tau, kernel, ssgsea.norm, verbose, BPPARAM)
+  
+  rval
+})
+
+setMethod("gsva", signature(expr="SingleCellExperiment", gset.idx.list="list"),
+          function(expr, gset.idx.list, annotation,
+  method=c("gsva", "ssgsea", "zscore", "plage"),
+  kcdf=c("Gaussian", "Poisson", "none"),
+  abs.ranking=FALSE,
+  min.sz=1,
+  max.sz=Inf,
+  parallel.sz=1L, 
+  mx.diff=TRUE,
+  tau=switch(method, gsva=1, ssgsea=0.25, NA),
+  ssgsea.norm=TRUE,
+  verbose=TRUE,
+  BPPARAM=SerialParam(progressbar=verbose))
+{
+  method <- match.arg(method)
+  kcdf <- match.arg(kcdf)
+  
+  warning("Using 'SingleCellExperiment' objects as input is still in an experimental stage.")
+
+  if (length(assays(expr)) == 0L)
+    stop("The input SummarizedExperiment object has no assay data.")
+  
+  se <- expr
+  
+  if (missing(annotation))
+    annotation <- names(assays(se))[1]
+  else {
+    if (!is.character(annotation))
+      stop("The 'annotation' argument must contain a character string.")
+    annotation <- annotation[1]
+    
+    if (!annotation %in% names(assays(se)))
+      stop(sprintf("Assay %s not found in the input SummarizedExperiment object.", annotation))
+  }
+  expr <- assays(se)[[annotation]]
+  
+  ## filter genes according to various criteria,
+  ## e.g., constant expression
+  
+  expr <- .filterFeatures(expr, method)
+  
+  ## map to the actual features for which expression data is available
+  mapped.gset.idx.list <- .mapGeneSetsToFeatures(gset.idx.list, rownames(expr))
+  
+  ## remove gene sets from the analysis for which no features are available
+  ## and meet the minimum and maximum gene-set size specified by the user
+  mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
+                                         min.sz=max(1, min.sz),
+                                         max.sz=max.sz)
+  
+  if (!missing(kcdf)) {
+    if (kcdf == "Gaussian") {
+      rnaseq <- FALSE
+      kernel <- TRUE
+    } else if (kcdf == "Poisson") {
+      rnaseq <- TRUE
+      kernel <- TRUE
+    } else
+      kernel <- FALSE
+  }
+  
+  eSco <- .gsva(expr, mapped.gset.idx.list, method, kcdf, rnaseq, abs.ranking,
+                parallel.sz, mx.diff, tau, kernel, ssgsea.norm, verbose, BPPARAM) 
+  
+  rval <- SingleCellExperiment(assays=SimpleList(es=eSco),
+                               colData=colData(se),
+                               metadata=metadata(se))
+  metadata(rval)$annotation <- NULL
+  
+  rval
+          })
+          
+setMethod("gsva", signature(expr="dgCMatrix", gset.idx.list="list"),
+          function(expr, gset.idx.list, annotation,
+  method=c("gsva", "ssgsea", "zscore", "plage"),
+  kcdf=c("Gaussian", "Poisson", "none"),
+  abs.ranking=FALSE,
+  min.sz=1,
+  max.sz=Inf,
+  parallel.sz=1L,
+  mx.diff=TRUE,
+  tau=switch(method, gsva=1, ssgsea=0.25, NA),
+  ssgsea.norm=TRUE,
+  verbose=TRUE,
+  BPPARAM=SerialParam(progressbar=verbose))
+{
+  method <- match.arg(method)
+  kcdf <- match.arg(kcdf)
+  
+  warning("Using 'dgCMatrix' objects as input is still in an experimental stage.")
+
+  ## filter genes according to verious criteria,
+  ## e.g., constant expression
+  expr <- .filterFeatures(expr, method)
+  
+  ## map to the actual features for which expression data is available
+  mapped.gset.idx.list <- .mapGeneSetsToFeatures(gset.idx.list, rownames(expr))
+  
+  ## remove gene sets from the analysis for which no features are available
+  ## and meet the minimum and maximum gene-set size specified by the user
+  mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
+                                         min.sz=max(1, min.sz),
+                                         max.sz=max.sz)
+  
+  if (!missing(kcdf)) {
+    if (kcdf == "Gaussian") {
+      rnaseq <- FALSE
+      kernel <- TRUE
+    } else if (kcdf == "Poisson") {
+      rnaseq <- TRUE
+      kernel <- TRUE
+    } else
+      kernel <- FALSE
+  }
+  
+  rval <- .gsva(expr, mapped.gset.idx.list, method, kcdf, rnaseq, abs.ranking,
+                parallel.sz, mx.diff, tau, kernel, ssgsea.norm, verbose, BPPARAM)
+  
+  rval
+})
+
 setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="GeneSetCollection"),
           function(expr, gset.idx.list, annotation,
   method=c("gsva", "ssgsea", "zscore", "plage"),
@@ -42,13 +212,10 @@ setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="GeneSetC
   ## e.g., constant expression
   expr <- .filterFeatures(expr, method)
 
-  if (nrow(expr) < 2)
-    stop("Less than two genes in the input ExpressionSet object\n")
-
   annotpkg <- metadata(se)$annotation
   if (!is.null(annotpkg) && length(annotpkg) > 0 && is.character(annotpkg) && annotpkg != "") {
-    if (!annotpkg %in% installed.packages())
-      stop(sprintf("Please install the nnotation package %s", annotpkg))
+    if (all(!c(annotpkg, paste0(annotpkg, ".db")) %in% installed.packages()))
+      stop(sprintf("Please install the annotation package %s. If %s does not seem to exist as a package, please try to append the suffix .db to its name.", annotpkg, annotpkg))
 
     if (verbose)
       cat("Mapping identifiers between gene sets and feature names\n")
@@ -58,7 +225,6 @@ setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="GeneSetC
     ## 'annotation' argument
     mapped.gset.idx.list <- mapIdentifiers(gset.idx.list,
                                            AnnoOrEntrezIdentifier(annotpkg))
-    mapped.gset.idx.list <- geneIds(mapped.gset.idx.list) 
   } else {
     mapped.gset.idx.list <- gset.idx.list
     if (verbose) {
@@ -66,6 +232,7 @@ setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="GeneSetC
           "Attempting to directly match identifiers in 'expr' to gene sets.", sep="\n")
     }
   }
+  mapped.gset.idx.list <- geneIds(mapped.gset.idx.list) 
 
   ## map to the actual features for which expression data is available
   mapped.gset.idx.list <- .mapGeneSetsToFeatures(mapped.gset.idx.list, rownames(expr))
@@ -135,12 +302,9 @@ setMethod("gsva", signature(expr="SummarizedExperiment", gset.idx.list="list"),
   ## e.g., constant expression
   expr <- .filterFeatures(expr, method)
 
-  if (nrow(expr) < 2)
-    stop("Less than two genes in the input ExpressionSet object\n")
-
   ## map to the actual features for which expression data is available
   mapped.gset.idx.list <- .mapGeneSetsToFeatures(gset.idx.list, rownames(expr))
-
+  
   ## remove gene sets from the analysis for which no features are available
   ## and meet the minimum and maximum gene-set size specified by the user
   mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
@@ -192,12 +356,9 @@ setMethod("gsva", signature(expr="ExpressionSet", gset.idx.list="list"),
   ## e.g., constant expression
   expr <- .filterFeatures(expr, method)
 
-  if (nrow(expr) < 2)
-    stop("Less than two genes in the input ExpressionSet object\n")
-
   ## map to the actual features for which expression data is available
   mapped.gset.idx.list <- .mapGeneSetsToFeatures(gset.idx.list, rownames(expr))
-
+  
   ## remove gene sets from the analysis for which no features are available
   ## and meet the minimum and maximum gene-set size specified by the user
   mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
@@ -247,13 +408,10 @@ setMethod("gsva", signature(expr="ExpressionSet", gset.idx.list="GeneSetCollecti
   ## e.g., constant expression
   expr <- .filterFeatures(expr, method)
 
-  if (nrow(expr) < 2)
-    stop("Less than two genes in the input ExpressionSet object\n")
-
   annotpkg <- Biobase::annotation(eset)
   if (length(annotpkg) > 0 && annotpkg != "") {
-    if (!annotpkg %in% installed.packages())
-      stop(sprintf("Please install the nnotation package %s", annotpkg))
+    if (all(!c(annotpkg, paste0(annotpkg, ".db")) %in% installed.packages()))
+      stop(sprintf("Please install the annotation package %s. If %s does not seem to exist as a package, please try to append the suffix .db to its name.", annotpkg, annotpkg))
 
     if (verbose)
       cat("Mapping identifiers between gene sets and feature names\n")
@@ -274,7 +432,7 @@ setMethod("gsva", signature(expr="ExpressionSet", gset.idx.list="GeneSetCollecti
 
   ## map to the actual features for which expression data is available
   mapped.gset.idx.list <- .mapGeneSetsToFeatures(mapped.gset.idx.list, rownames(expr))
-
+  
   ## remove gene sets from the analysis for which no features are available
   ## and meet the minimum and maximum gene-set size specified by the user
   mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
@@ -322,9 +480,6 @@ setMethod("gsva", signature(expr="matrix", gset.idx.list="GeneSetCollection"),
   ## e.g., constant expression
   expr <- .filterFeatures(expr, method)
 
-  if (nrow(expr) < 2)
-    stop("Less than two genes in the input expression data matrix\n")
-
   ## map gene identifiers of the gene sets to the features in the matrix
   mapped.gset.idx.list <- gset.idx.list
   if (!missing(annotation)) {
@@ -338,7 +493,7 @@ setMethod("gsva", signature(expr="matrix", gset.idx.list="GeneSetCollection"),
   
   ## map to the actual features for which expression data is available
   mapped.gset.idx.list <- .mapGeneSetsToFeatures(mapped.gset.idx.list, rownames(expr))
-
+  
   ## remove gene sets from the analysis for which no features are available
   ## and meet the minimum and maximum gene-set size specified by the user
   mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
@@ -384,12 +539,9 @@ setMethod("gsva", signature(expr="matrix", gset.idx.list="list"),
   ## e.g., constant expression
   expr <- .filterFeatures(expr, method)
 
-  if (nrow(expr) < 2)
-    stop("Less than two genes in the input expression data matrix\n")
-
   ## map to the actual features for which expression data is available
   mapped.gset.idx.list <- .mapGeneSetsToFeatures(gset.idx.list, rownames(expr))
-
+  
   ## remove gene sets from the analysis for which no features are available
   ## and meet the minimum and maximum gene-set size specified by the user
   mapped.gset.idx.list <- filterGeneSets(mapped.gset.idx.list,
@@ -425,6 +577,13 @@ setMethod("gsva", signature(expr="matrix", gset.idx.list="list"),
   ssgsea.norm=TRUE,
   verbose=TRUE,
   BPPARAM=SerialParam(progressbar=verbose)) {
+  
+  if(is(expr, "DelayedArray")){
+    warning("Using 'DelayedArray' objects as input is still in an experimental stage.")
+
+    return(.gsvaDelayedArray(expr, gset.idx.list, method, kcdf, rnaseq, abs.ranking,
+                             parallel.sz, mx.diff, tau, kernel, ssgsea.norm, verbose, BPPARAM))
+  }
 
 	if (length(gset.idx.list) == 0)
 		stop("The gene set list is empty! Filter may be too stringent.")
@@ -549,7 +708,7 @@ compute.geneset.es <- function(expr, gset.idx.list, sample.idxs, rnaseq=FALSE,
   ## more than 100 genes on more than 100 samples
   if (parallel.sz > 1 && length(sample.idxs > 100) && nrow(expr) > 100) {
     if (verbose)
-      cat(sprintf("Estimating ECDFs in parallel\n", parallel.sz))
+      cat(sprintf("Estimating ECDFs in parallel on %d cores\n", as.integer(parallel.sz)))
     iter <- function(Y, n_chunks=BiocParallel::multicoreWorkers()) {
       idx <- splitIndices(nrow(Y), min(nrow(Y), n_chunks))
       i <- 0L
@@ -691,37 +850,43 @@ ssgsea <- function(X, geneSets, alpha=0.25, parallel.sz,
                    normalization=TRUE, verbose=TRUE,
                    BPPARAM=SerialParam(progressbar=verbose)) {
 
-  p <- nrow(X)
   n <- ncol(X)
-
-  R <- apply(X, 2, function(x, p) as.integer(rank(x)), p)
+  
+  if(is(X, "dgCMatrix")){
+    R <- t(sparseMatrixStats::colRanks(X, ties.method = "average"))
+    mode(R) <- "integer"
+  } else {
+    R <- apply(X, 2, function(x, p) as.integer(rank(x)))
+  }
+  
   Ra <- abs(R)^alpha
-
-  es <- matrix(NA, nrow=length(geneSets), ncol=ncol(X))
-
+  
   es <- bplapply(as.list(1:n), function(j) {
-                     geneRanking <- order(R[, j], decreasing=TRUE)
-                     es_sample <- lapply(geneSets, .fastRndWalk, geneRanking, j, Ra)
-
-                     unlist(es_sample)
-                   }, BPPARAM=BPPARAM)
+    geneRanking <- order(R[, j], decreasing=TRUE)
+    es_sample <- lapply(geneSets, .fastRndWalk, geneRanking, j, Ra)
+    
+    unlist(es_sample)
+  }, BPPARAM=BPPARAM)
+  
   es <- do.call("cbind", es)
-
-  if (length(geneSets) == 1)
-    es <- matrix(es, nrow=1)
-
+  
+  
   if (normalization) {
     ## normalize enrichment scores by using the entire data set, as indicated
     ## by Barbie et al., 2009, online methods, pg. 2
     es <- apply(es, 2, function(x, es) x / (range(es)[2] - range(es)[1]), es)
   }
-
+  
   if (length(geneSets) == 1)
     es <- matrix(es, nrow=1)
-
+  
   rownames(es) <- names(geneSets)
   colnames(es) <- colnames(X)
-
+  
+  if(is(X, "dgCMatrix")){
+    es <- as(es, "dgCMatrix")
+  }
+  
   es
 }
 
@@ -730,58 +895,83 @@ combinez <- function(gSetIdx, j, Z) sum(Z[gSetIdx, j]) / sqrt(length(gSetIdx))
 zscore <- function(X, geneSets, parallel.sz, verbose=TRUE,
                    BPPARAM=SerialParam(progressbar=verbose)) {
 
-  p <- nrow(X)
   n <- ncol(X)
 
-  Z <- t(scale(t(X)))
-
-  es <- matrix(NA, nrow=length(geneSets), ncol=ncol(X))
+  if(is(X, "dgCMatrix")){
+    message("Please bear in mind that this method first scales the values of the gene
+    expression data. In order to take advantage of the sparse Matrix type, the scaling
+    will only be applied to the non-zero values of the data. This is a provisional 
+    solution in order to give support to the dgCMatrix format.")
+    
+    Z <- t(X)
+    Z <- .dgCapply(Z, scale, 2)
+    Z <- t(Z)
+    
+  } else {
+    Z <- t(scale(t(X)))
+  }
 
   es <- bplapply(as.list(1:n), function(j, Z, geneSets) {
-                   es_sample <- lapply(geneSets, combinez, j, Z)
+    es_sample <- lapply(geneSets, combinez, j, Z)
+    unlist(es_sample)
+  }, Z, geneSets, BPPARAM=BPPARAM)
 
-                   unlist(es_sample)
-                 }, Z, geneSets, BPPARAM=BPPARAM)
-    es <- do.call("cbind", es)
-
-  if (length(geneSets) == 1)
-    es <- matrix(es, nrow=1)
+  es <- do.call("cbind", es)
+  
+  if(is(X, "dgCMatrix")){
+    es <- as(es, "dgCMatrix")
+  }
 
   rownames(es) <- names(geneSets)
   colnames(es) <- colnames(X)
 
-  if (verbose && length(geneSets) > n) {
-    setTxtProgressBar(get("progressBar", envir=globalenv()), 1)
-    close(get("progressBar", envir=globalenv()))
-  }
-
   es
 }
 
 rightsingularsvdvectorgset <- function(gSetIdx, Z) {
+  if(is(Z, "dgCMatrix")){
+    s <- BiocSingular::runExactSVD(Z[gSetIdx, ])
+  } else {
     s <- svd(Z[gSetIdx, ])
+  }
   s$v[, 1]
 }
 
 plage <- function(X, geneSets, parallel.sz, verbose=TRUE,
                   BPPARAM=SerialParam(progressbar=verbose)) {
-
-  p <- nrow(X)
-  n <- ncol(X)
-
-  Z <- t(scale(t(X)))
-
-  es <- bplapply(geneSets, rightsingularsvdvectorgset, Z,
-                 BPPARAM=BPPARAM)
-
-  es <- do.call(rbind, es)
-
-  if (length(geneSets) == 1)
-    es <- matrix(es, nrow=1)
-
-  rownames(es) <- names(geneSets)
-  colnames(es) <- colnames(X)
-
+  if(is(X, "dgCMatrix")){
+    message("Please bear in mind that this method first scales the values of the gene
+    expression data. In order to take advantage of the sparse Matrix type, the scaling
+    will only be applied to the non-zero values of the data. This is a provisional 
+    solution in order to give support to the dgCMatrix format.")
+    
+    Z <- Matrix::t(X)
+    Z <- .dgCapply(Z, scale, 2)
+    Z <- Matrix::t(Z)
+    
+    es <- bplapply(geneSets, rightsingularsvdvectorgset, Z,
+                   BPPARAM=BPPARAM)
+    
+    es <- do.call(rbind, es)
+    
+    es <- as(es, "dgCMatrix")
+    
+  } else {
+    
+    Z <- t(scale(t(X)))
+    
+    es <- bplapply(geneSets, rightsingularsvdvectorgset, Z,
+                   BPPARAM=BPPARAM)
+    
+    es <- do.call(rbind, es)
+    
+    if (length(geneSets) == 1)
+      es <- matrix(es, nrow=1)
+    
+    rownames(es) <- names(geneSets)
+    colnames(es) <- colnames(X)
+  }
+  
   es
 }
 
@@ -789,14 +979,13 @@ setGeneric("filterGeneSets", function(gSets, ...) standardGeneric("filterGeneSet
 
 setMethod("filterGeneSets", signature(gSets="list"),
           function(gSets, min.sz=1, max.sz=Inf) {
-	gSetsLen <- sapply(gSets,length)
+	gSetsLen <- lengths(gSets)
 	return (gSets[gSetsLen >= min.sz & gSetsLen <= max.sz])	
 })
 
 setMethod("filterGeneSets", signature(gSets="GeneSetCollection"),
           function(gSets, min.sz=1, max.sz=Inf) {
-	gSetsLen <- sapply(geneIds(gSets),length)
-	return (gSets[gSetsLen >= min.sz & gSetsLen <= max.sz])	
+  filterGeneSets(geneIds(gSets), min.sz, max.sz)
 })
 
 
@@ -810,7 +999,7 @@ setMethod("computeGeneSetsOverlap", signature(gSets="list", uniqGenes="character
   ## map to the actual features for which expression data is available
   gSets <- .mapGeneSetsToFeatures(gSets, uniqGenes)
 
-  lenGsets <- sapply(gSets, length)
+  lenGsets <- lengths(gSets)
   totalGsets <- length(gSets)
 
   gSetsMembershipMatrix <- matrix(0, nrow=totalGenes, ncol=totalGsets,
@@ -829,7 +1018,7 @@ setMethod("computeGeneSetsOverlap", signature(gSets="list", uniqGenes="Expressio
   ## map to the actual features for which expression data is available
   gSets <- .mapGeneSetsToFeatures(gSets, uniqGenes)
 
-  lenGsets <- sapply(gSets, length)
+  lenGsets <- lengths(gSets)
   totalGsets <- length(gSets)
 
   gSetsMembershipMatrix <- matrix(0, nrow=totalGenes, ncol=totalGsets,
